==> cosmosmash-1.4.2.tar.gz: cosmosmash-1.4.2/src/AutoController.h <==
/*  $Id: AutoController.h,v 1.1 2007/06/19 23:26:46 sarrazip Exp $
    AutoController.h - Automatic game controller

    cosmosmash - A space rock shooting video game.
    Copyright (C) 2007 Pierre Sarrazin <http://sarrazip.com/>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
    02111-1307, USA.
*/

#ifndef _H_AutoController
#define _H_AutoController

#include "Controller.h"

#include <stdlib.h>
#include <limits.h>


class AutoController : public Controller
{
public:
    AutoController()
      : requestStart(true),
	closestRockDist(HUGE_VAL),
	goLeft(false),
	goRight(false),
	goHyperspace(false)
    {
    }

    /** Indicates if the player has just asked to start a game.
	Returns true if yes, and then 'isExtendedMode' indicates
	if the non-Intellivision extensions must be activated.
	Otherwise, returns false and 'isExtendedMode' is false.
    */
    virtual bool isStartRequested(bool &isExtendedMode)
    {
	isExtendedMode = false;
	bool ret = requestStart;
	requestStart = false;
	return ret;
    }

    virtual bool isPauseRequested()
    {
	return false;
    }

    virtual bool isResumeRequested()
    {
	return false;
    }

    virtual bool isShootingActive()
    {
	return true;
    }

    virtual bool isHyperspaceRequested()
    {
	return goHyperspace;
    }

    virtual bool isLeftMoveRequested()
    {
	return goLeft;
    }

    virtual bool isRightMoveRequested()
    {
	return goRight;
    }

    /** Do any processing that needs to be done at the end of
	a tick.  In this case, we remember the state of some
	keys (pressed or not), so that justPressed() will work
	in the next tick.
    */
    virtual void endOfTick()
    {
    }

    void beginGameState()
    {
	closestRockDist = HUGE_VAL;
	closestRockPos = Couple(-1, -1);

	lowestSpinnerPos = Couple(-1, -1);

	closestPulsarDist = HUGE_VAL;
	closestPulsarPos = Couple(-1, -1);
    }

    void setPlayerPos(Couple pos)
    {
	playerPos = pos;
    }

    void addRockPos(Couple pos)
    {
	Couple delta = playerPos - pos;
	double thisDist = delta.length();
	if (thisDist < closestRockDist)
	{
	    closestRockDist = thisDist;
	    closestRockPos = pos;
	}
    }

    void addSpinnerPos(Couple pos)
    {
	// The automated player can sometimes start oscillating
	// below a small spinnier that falls vertically.
	// This patch introduces a bit of randomness to help
	// avoid these situations.
	pos.x += rand() % 11 - 5;

	if (pos.y > lowestSpinnerPos.y)
	    lowestSpinnerPos = pos;
    }

    void addPulsarPos(Couple pos)
    {
	Couple delta = playerPos - pos;
	double thisDist = delta.length();
	if (thisDist < closestPulsarDist)
	{
	    closestPulsarDist = thisDist;
	    closestPulsarPos = pos;
	}
    }

    void endGameState()
    {
	goLeft = goRight = goHyperspace = false;

	// If a pulsar is very close, use hyperspace.
	if (closestPulsarDist < 100)
	    goHyperspace = true;

	// Move if a rock is very close and at a side.
	if (closestRockPos.y >= 0 && closestRockDist < 100)
	{
	    goLeft  = playerPos.x <= closestRockPos.x;
	    goRight = !goLeft;
	    return;
	}

	// Move if there is at least one spinner.
	if (lowestSpinnerPos.y >= 0)
	{
	    goLeft  = playerPos.x > lowestSpinnerPos.x;
	    goRight = playerPos.x < lowestSpinnerPos.x;
	    return;
	}

	// Assume there is no imminent danger,
	// so pursue the nearest rock.
	goLeft  = closestRockPos.y >= 0 && playerPos.x > closestRockPos.x;
	goRight = closestRockPos.y >= 0 && playerPos.x < closestRockPos.x;
    }


private:

    bool requestStart;

    Couple playerPos;

    Couple closestRockPos;  // y == -1 means no rock
    double closestRockDist;

    Couple lowestSpinnerPos;  // y == -1 means no spinner

    Couple closestPulsarPos;  // y == -1 means no rock
    double closestPulsarDist;

    bool goLeft;
    bool goRight;
    bool goHyperspace;
};


#endif  /* _H_AutoController */

==> cosmosmash-1.4.3.tar.gz: cosmosmash-1.4.3/src/AutoController.h <==
/*  $Id: AutoController.h,v 1.5 2007/06/26 02:55:04 sarrazip Exp $
    AutoController.h - Automatic game controller

    cosmosmash - A space rock shooting video game.
    Copyright (C) 2007 Pierre Sarrazin <http://sarrazip.com/>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
    02111-1307, USA.
*/

#ifndef _H_AutoController
#define _H_AutoController

#include "Controller.h"

#include <stdlib.h>
#include <limits.h>


class AutoController : public Controller
{
public:
    AutoController()
      : requestStart(true),
	closestRockDist(HUGE_VAL),
	goLeft(false),
	goRight(false),
	goHyperspace(false)
    {
    }

    /** Indicates if the player has just asked to start a game.
	Returns true if yes, and then 'isExtendedMode' indicates
	if the non-Intellivision extensions must be activated.
	Otherwise, returns false and 'isExtendedMode' is false.
    */
    virtual bool isStartRequested(bool &isExtendedMode)
    {
	isExtendedMode = false;
	bool ret = requestStart;
	requestStart = false;
	return ret;
    }

    virtual bool isShootingActive()
    {
	return true;
    }

    virtual bool isHyperspaceRequested()
    {
	return goHyperspace;
    }

    virtual bool isLeftMoveRequested()
    {
	return goLeft;
    }

    virtual bool isRightMoveRequested()
    {
	return goRight;
    }

    virtual void beginGameState()
    {
	closestRockDist = HUGE_VAL;
	closestRockPos = flatzebra::Couple(-1, -1);

	lowestSpinnerPos = flatzebra::Couple(-1, -1);

	closestPulsarDist = HUGE_VAL;
	closestPulsarPos = flatzebra::Couple(-1, -1);
    }

    virtual void setPlayerPos(flatzebra::Couple pos)
    {
	playerPos = pos;
    }

    virtual void addRockPos(flatzebra::Couple pos)
    {
	flatzebra::Couple delta = playerPos - pos;
	double thisDist = delta.length();
	if (thisDist < closestRockDist)
	{
	    closestRockDist = thisDist;
	    closestRockPos = pos;
	}
    }

    virtual void addSpinnerPos(flatzebra::Couple pos)
    {
	// The automated player can sometimes start oscillating
	// below a small spinnier that falls vertically.
	// This patch introduces a bit of randomness to help
	// avoid these situations.
	pos.x += rand() % 11 - 5;

	if (pos.y > lowestSpinnerPos.y)
	    lowestSpinnerPos = pos;
    }

    virtual void addPulsarPos(flatzebra::Couple pos)
    {
	flatzebra::Couple delta = playerPos - pos;
	double thisDist = delta.length();
	if (thisDist < closestPulsarDist)
	{
	    closestPulsarDist = thisDist;
	    closestPulsarPos = pos;
	}
    }

    virtual void endGameState()
    {
	goLeft = goRight = goHyperspace = false;

	// If a pulsar is very close, use hyperspace.
	if (closestPulsarDist < 100)
	    goHyperspace = true;

	// Move if a rock is very close and at a side.
	if (closestRockPos.y >= 0 && closestRockDist < 100)
	{
	    goLeft  = playerPos.x <= closestRockPos.x;
	    goRight = !goLeft;
	    return;
	}

	// Move if there is at least one spinner.
	if (lowestSpinnerPos.y >= 0)
	{
	    goLeft  = playerPos.x > lowestSpinnerPos.x;
	    goRight = playerPos.x < lowestSpinnerPos.x;
	    return;
	}

	// Assume there is no imminent danger,
	// so pursue the nearest rock.
	goLeft  = closestRockPos.y >= 0 && playerPos.x > closestRockPos.x;
	goRight = closestRockPos.y >= 0 && playerPos.x < closestRockPos.x;
    }


private:

    bool requestStart;

    flatzebra::Couple playerPos;

    flatzebra::Couple closestRockPos;  // y == -1 means no rock
    double closestRockDist;

    flatzebra::Couple lowestSpinnerPos;  // y == -1 means no spinner

    flatzebra::Couple closestPulsarPos;  // y == -1 means no rock
    double closestPulsarDist;

    bool goLeft;
    bool goRight;
    bool goHyperspace;
};


#endif  /* _H_AutoController */
